// Copyright (c) 2016, 2018, 2023, Oracle and/or its affiliates.  All rights reserved.
// This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.
// Code generated. DO NOT EDIT.

package cloudguard

import (
	"fmt"
	"github.com/oracle/oci-go-sdk/v65/common"
	"net/http"
	"strings"
)

// ListImpactedResourcesRequest wrapper for the ListImpactedResources operation
//
// # See also
//
// Click https://docs.cloud.oracle.com/en-us/iaas/tools/go-sdk-examples/latest/cloudguard/ListImpactedResources.go.html to see an example of how to use ListImpactedResourcesRequest.
type ListImpactedResourcesRequest struct {

	// OCId of the problem.
	ProblemId *string `mandatory:"true" contributesTo:"path" name:"problemId"`

	// The maximum number of items to return.
	Limit *int `mandatory:"false" contributesTo:"query" name:"limit"`

	// The page token representing the page at which to start retrieving results. This is usually retrieved from a previous list call.
	Page *string `mandatory:"false" contributesTo:"query" name:"page"`

	// The sort order to use, either 'asc' or 'desc'.
	SortOrder ListImpactedResourcesSortOrderEnum `mandatory:"false" contributesTo:"query" name:"sortOrder" omitEmpty:"true"`

	// The field to sort by. Only one sort order may be provided. Default order for timeCreated is descending. If no value is specified timeCreated is default.
	SortBy ListImpactedResourcesSortByEnum `mandatory:"false" contributesTo:"query" name:"sortBy" omitEmpty:"true"`

	// The client request ID for tracing.
	OpcRequestId *string `mandatory:"false" contributesTo:"header" name:"opc-request-id"`

	// Metadata about the request. This information will not be transmitted to the service, but
	// represents information that the SDK will consume to drive retry behavior.
	RequestMetadata common.RequestMetadata
}

func (request ListImpactedResourcesRequest) String() string {
	return common.PointerString(request)
}

// HTTPRequest implements the OCIRequest interface
func (request ListImpactedResourcesRequest) HTTPRequest(method, path string, binaryRequestBody *common.OCIReadSeekCloser, extraHeaders map[string]string) (http.Request, error) {

	_, err := request.ValidateEnumValue()
	if err != nil {
		return http.Request{}, err
	}
	return common.MakeDefaultHTTPRequestWithTaggedStructAndExtraHeaders(method, path, request, extraHeaders)
}

// BinaryRequestBody implements the OCIRequest interface
func (request ListImpactedResourcesRequest) BinaryRequestBody() (*common.OCIReadSeekCloser, bool) {

	return nil, false

}

// RetryPolicy implements the OCIRetryableRequest interface. This retrieves the specified retry policy.
func (request ListImpactedResourcesRequest) RetryPolicy() *common.RetryPolicy {
	return request.RequestMetadata.RetryPolicy
}

// ValidateEnumValue returns an error when providing an unsupported enum value
// This function is being called during constructing API request process
// Not recommended for calling this function directly
func (request ListImpactedResourcesRequest) ValidateEnumValue() (bool, error) {
	errMessage := []string{}
	if _, ok := GetMappingListImpactedResourcesSortOrderEnum(string(request.SortOrder)); !ok && request.SortOrder != "" {
		errMessage = append(errMessage, fmt.Sprintf("unsupported enum value for SortOrder: %s. Supported values are: %s.", request.SortOrder, strings.Join(GetListImpactedResourcesSortOrderEnumStringValues(), ",")))
	}
	if _, ok := GetMappingListImpactedResourcesSortByEnum(string(request.SortBy)); !ok && request.SortBy != "" {
		errMessage = append(errMessage, fmt.Sprintf("unsupported enum value for SortBy: %s. Supported values are: %s.", request.SortBy, strings.Join(GetListImpactedResourcesSortByEnumStringValues(), ",")))
	}
	if len(errMessage) > 0 {
		return true, fmt.Errorf(strings.Join(errMessage, "\n"))
	}
	return false, nil
}

// ListImpactedResourcesResponse wrapper for the ListImpactedResources operation
type ListImpactedResourcesResponse struct {

	// The underlying http response
	RawResponse *http.Response

	// A list of ImpactedResourceCollection instances
	ImpactedResourceCollection `presentIn:"body"`

	// Unique Oracle-assigned identifier for the request. If you need to contact
	// Oracle about a particular request, please provide the request ID.
	OpcRequestId *string `presentIn:"header" name:"opc-request-id"`

	// For pagination of a list of items. When paging through a list, if this header appears in the response,
	// then a partial list might have been returned. Include this value as the `page` parameter for the
	// subsequent GET request to get the next batch of items.
	OpcNextPage *string `presentIn:"header" name:"opc-next-page"`
}

func (response ListImpactedResourcesResponse) String() string {
	return common.PointerString(response)
}

// HTTPResponse implements the OCIResponse interface
func (response ListImpactedResourcesResponse) HTTPResponse() *http.Response {
	return response.RawResponse
}

// ListImpactedResourcesSortOrderEnum Enum with underlying type: string
type ListImpactedResourcesSortOrderEnum string

// Set of constants representing the allowable values for ListImpactedResourcesSortOrderEnum
const (
	ListImpactedResourcesSortOrderAsc  ListImpactedResourcesSortOrderEnum = "ASC"
	ListImpactedResourcesSortOrderDesc ListImpactedResourcesSortOrderEnum = "DESC"
)

var mappingListImpactedResourcesSortOrderEnum = map[string]ListImpactedResourcesSortOrderEnum{
	"ASC":  ListImpactedResourcesSortOrderAsc,
	"DESC": ListImpactedResourcesSortOrderDesc,
}

var mappingListImpactedResourcesSortOrderEnumLowerCase = map[string]ListImpactedResourcesSortOrderEnum{
	"asc":  ListImpactedResourcesSortOrderAsc,
	"desc": ListImpactedResourcesSortOrderDesc,
}

// GetListImpactedResourcesSortOrderEnumValues Enumerates the set of values for ListImpactedResourcesSortOrderEnum
func GetListImpactedResourcesSortOrderEnumValues() []ListImpactedResourcesSortOrderEnum {
	values := make([]ListImpactedResourcesSortOrderEnum, 0)
	for _, v := range mappingListImpactedResourcesSortOrderEnum {
		values = append(values, v)
	}
	return values
}

// GetListImpactedResourcesSortOrderEnumStringValues Enumerates the set of values in String for ListImpactedResourcesSortOrderEnum
func GetListImpactedResourcesSortOrderEnumStringValues() []string {
	return []string{
		"ASC",
		"DESC",
	}
}

// GetMappingListImpactedResourcesSortOrderEnum performs case Insensitive comparison on enum value and return the desired enum
func GetMappingListImpactedResourcesSortOrderEnum(val string) (ListImpactedResourcesSortOrderEnum, bool) {
	enum, ok := mappingListImpactedResourcesSortOrderEnumLowerCase[strings.ToLower(val)]
	return enum, ok
}

// ListImpactedResourcesSortByEnum Enum with underlying type: string
type ListImpactedResourcesSortByEnum string

// Set of constants representing the allowable values for ListImpactedResourcesSortByEnum
const (
	ListImpactedResourcesSortByTimecreated ListImpactedResourcesSortByEnum = "timeCreated"
)

var mappingListImpactedResourcesSortByEnum = map[string]ListImpactedResourcesSortByEnum{
	"timeCreated": ListImpactedResourcesSortByTimecreated,
}

var mappingListImpactedResourcesSortByEnumLowerCase = map[string]ListImpactedResourcesSortByEnum{
	"timecreated": ListImpactedResourcesSortByTimecreated,
}

// GetListImpactedResourcesSortByEnumValues Enumerates the set of values for ListImpactedResourcesSortByEnum
func GetListImpactedResourcesSortByEnumValues() []ListImpactedResourcesSortByEnum {
	values := make([]ListImpactedResourcesSortByEnum, 0)
	for _, v := range mappingListImpactedResourcesSortByEnum {
		values = append(values, v)
	}
	return values
}

// GetListImpactedResourcesSortByEnumStringValues Enumerates the set of values in String for ListImpactedResourcesSortByEnum
func GetListImpactedResourcesSortByEnumStringValues() []string {
	return []string{
		"timeCreated",
	}
}

// GetMappingListImpactedResourcesSortByEnum performs case Insensitive comparison on enum value and return the desired enum
func GetMappingListImpactedResourcesSortByEnum(val string) (ListImpactedResourcesSortByEnum, bool) {
	enum, ok := mappingListImpactedResourcesSortByEnumLowerCase[strings.ToLower(val)]
	return enum, ok
}
