// Copyright (c) 2016, 2018, 2023, Oracle and/or its affiliates.  All rights reserved.
// This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.
// Code generated. DO NOT EDIT.

// Database Management API
//
// Use the Database Management API to perform tasks such as obtaining performance and resource usage metrics
// for a fleet of Managed Databases or a specific Managed Database, creating Managed Database Groups, and
// running a SQL job on a Managed Database or Managed Database Group.
//

package databasemanagement

import (
	"encoding/json"
	"fmt"
	"github.com/oracle/oci-go-sdk/v65/common"
	"strings"
)

// DiscoveredExternalDbHome The details of an Oracle DB home discovered in an external DB system discovery run.
type DiscoveredExternalDbHome struct {

	// The identifier of the discovered DB system component.
	ComponentId *string `mandatory:"true" json:"componentId"`

	// The user-friendly name for the discovered DB system component. The name does not have to be unique.
	DisplayName *string `mandatory:"true" json:"displayName"`

	// The name of the discovered DB system component.
	ComponentName *string `mandatory:"true" json:"componentName"`

	// The location of the DB home.
	HomeDirectory *string `mandatory:"true" json:"homeDirectory"`

	// The OCID (https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the existing OCI resource matching the discovered DB system component.
	ResourceId *string `mandatory:"false" json:"resourceId"`

	// Indicates whether the DB system component should be provisioned as an OCI resource or not.
	IsSelectedForMonitoring *bool `mandatory:"false" json:"isSelectedForMonitoring"`

	// The list of associated components.
	AssociatedComponents []AssociatedComponent `mandatory:"false" json:"associatedComponents"`

	// The state of the discovered DB system component.
	Status DiscoveredExternalDbSystemComponentStatusEnum `mandatory:"false" json:"status,omitempty"`
}

//GetComponentId returns ComponentId
func (m DiscoveredExternalDbHome) GetComponentId() *string {
	return m.ComponentId
}

//GetDisplayName returns DisplayName
func (m DiscoveredExternalDbHome) GetDisplayName() *string {
	return m.DisplayName
}

//GetComponentName returns ComponentName
func (m DiscoveredExternalDbHome) GetComponentName() *string {
	return m.ComponentName
}

//GetResourceId returns ResourceId
func (m DiscoveredExternalDbHome) GetResourceId() *string {
	return m.ResourceId
}

//GetIsSelectedForMonitoring returns IsSelectedForMonitoring
func (m DiscoveredExternalDbHome) GetIsSelectedForMonitoring() *bool {
	return m.IsSelectedForMonitoring
}

//GetStatus returns Status
func (m DiscoveredExternalDbHome) GetStatus() DiscoveredExternalDbSystemComponentStatusEnum {
	return m.Status
}

//GetAssociatedComponents returns AssociatedComponents
func (m DiscoveredExternalDbHome) GetAssociatedComponents() []AssociatedComponent {
	return m.AssociatedComponents
}

func (m DiscoveredExternalDbHome) String() string {
	return common.PointerString(m)
}

// ValidateEnumValue returns an error when providing an unsupported enum value
// This function is being called during constructing API request process
// Not recommended for calling this function directly
func (m DiscoveredExternalDbHome) ValidateEnumValue() (bool, error) {
	errMessage := []string{}

	if _, ok := GetMappingDiscoveredExternalDbSystemComponentStatusEnum(string(m.Status)); !ok && m.Status != "" {
		errMessage = append(errMessage, fmt.Sprintf("unsupported enum value for Status: %s. Supported values are: %s.", m.Status, strings.Join(GetDiscoveredExternalDbSystemComponentStatusEnumStringValues(), ",")))
	}
	if len(errMessage) > 0 {
		return true, fmt.Errorf(strings.Join(errMessage, "\n"))
	}
	return false, nil
}

// MarshalJSON marshals to json representation
func (m DiscoveredExternalDbHome) MarshalJSON() (buff []byte, e error) {
	type MarshalTypeDiscoveredExternalDbHome DiscoveredExternalDbHome
	s := struct {
		DiscriminatorParam string `json:"componentType"`
		MarshalTypeDiscoveredExternalDbHome
	}{
		"DATABASE_HOME",
		(MarshalTypeDiscoveredExternalDbHome)(m),
	}

	return json.Marshal(&s)
}
