// Copyright (c) 2016, 2018, 2023, Oracle and/or its affiliates.  All rights reserved.
// This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.
// Code generated. DO NOT EDIT.

// Database Management API
//
// Use the Database Management API to perform tasks such as obtaining performance and resource usage metrics
// for a fleet of Managed Databases or a specific Managed Database, creating Managed Database Groups, and
// running a SQL job on a Managed Database or Managed Database Group.
//

package databasemanagement

import (
	"encoding/json"
	"fmt"
	"github.com/oracle/oci-go-sdk/v65/common"
	"strings"
)

// ExternalStorageGridDiscoverySummary The summary of the Exadata storage server grid discovery.
type ExternalStorageGridDiscoverySummary struct {

	// The name of the entity.
	DisplayName *string `mandatory:"true" json:"displayName"`

	// The OCID (https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the entity discovered.
	Id *string `mandatory:"false" json:"id"`

	// The OCID (https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the agent used for monitoring.
	AgentId *string `mandatory:"false" json:"agentId"`

	// The OCID (https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the associated connector.
	ConnectorId *string `mandatory:"false" json:"connectorId"`

	// The version of the entity.
	Version *string `mandatory:"false" json:"version"`

	// The internal identifier of the entity.
	InternalId *string `mandatory:"false" json:"internalId"`

	// The status of the entity.
	Status *string `mandatory:"false" json:"status"`

	// The error code of the discovery.
	DiscoverErrorCode *string `mandatory:"false" json:"discoverErrorCode"`

	// The error message of the discovery.
	DiscoverErrorMsg *string `mandatory:"false" json:"discoverErrorMsg"`

	// The total number of Exadata storage servers discovered.
	CountOfStorageServersDiscovered *int `mandatory:"false" json:"countOfStorageServersDiscovered"`

	// The status of the entity discovery.
	DiscoverStatus EntityDiscoveredDiscoverStatusEnum `mandatory:"false" json:"discoverStatus,omitempty"`
}

//GetId returns Id
func (m ExternalStorageGridDiscoverySummary) GetId() *string {
	return m.Id
}

//GetAgentId returns AgentId
func (m ExternalStorageGridDiscoverySummary) GetAgentId() *string {
	return m.AgentId
}

//GetConnectorId returns ConnectorId
func (m ExternalStorageGridDiscoverySummary) GetConnectorId() *string {
	return m.ConnectorId
}

//GetDisplayName returns DisplayName
func (m ExternalStorageGridDiscoverySummary) GetDisplayName() *string {
	return m.DisplayName
}

//GetVersion returns Version
func (m ExternalStorageGridDiscoverySummary) GetVersion() *string {
	return m.Version
}

//GetInternalId returns InternalId
func (m ExternalStorageGridDiscoverySummary) GetInternalId() *string {
	return m.InternalId
}

//GetStatus returns Status
func (m ExternalStorageGridDiscoverySummary) GetStatus() *string {
	return m.Status
}

//GetDiscoverStatus returns DiscoverStatus
func (m ExternalStorageGridDiscoverySummary) GetDiscoverStatus() EntityDiscoveredDiscoverStatusEnum {
	return m.DiscoverStatus
}

//GetDiscoverErrorCode returns DiscoverErrorCode
func (m ExternalStorageGridDiscoverySummary) GetDiscoverErrorCode() *string {
	return m.DiscoverErrorCode
}

//GetDiscoverErrorMsg returns DiscoverErrorMsg
func (m ExternalStorageGridDiscoverySummary) GetDiscoverErrorMsg() *string {
	return m.DiscoverErrorMsg
}

func (m ExternalStorageGridDiscoverySummary) String() string {
	return common.PointerString(m)
}

// ValidateEnumValue returns an error when providing an unsupported enum value
// This function is being called during constructing API request process
// Not recommended for calling this function directly
func (m ExternalStorageGridDiscoverySummary) ValidateEnumValue() (bool, error) {
	errMessage := []string{}

	if _, ok := GetMappingEntityDiscoveredDiscoverStatusEnum(string(m.DiscoverStatus)); !ok && m.DiscoverStatus != "" {
		errMessage = append(errMessage, fmt.Sprintf("unsupported enum value for DiscoverStatus: %s. Supported values are: %s.", m.DiscoverStatus, strings.Join(GetEntityDiscoveredDiscoverStatusEnumStringValues(), ",")))
	}
	if len(errMessage) > 0 {
		return true, fmt.Errorf(strings.Join(errMessage, "\n"))
	}
	return false, nil
}

// MarshalJSON marshals to json representation
func (m ExternalStorageGridDiscoverySummary) MarshalJSON() (buff []byte, e error) {
	type MarshalTypeExternalStorageGridDiscoverySummary ExternalStorageGridDiscoverySummary
	s := struct {
		DiscriminatorParam string `json:"entityType"`
		MarshalTypeExternalStorageGridDiscoverySummary
	}{
		"STORAGE_GRID_DISCOVER_SUMMARY",
		(MarshalTypeExternalStorageGridDiscoverySummary)(m),
	}

	return json.Marshal(&s)
}
