// Copyright (c) 2016, 2018, 2023, Oracle and/or its affiliates.  All rights reserved.
// This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.
// Code generated. DO NOT EDIT.

package datasafe

import (
	"fmt"
	"github.com/oracle/oci-go-sdk/v65/common"
	"net/http"
	"strings"
)

// GetCompatibleFormatsForSensitiveTypesRequest wrapper for the GetCompatibleFormatsForSensitiveTypes operation
//
// # See also
//
// Click https://docs.cloud.oracle.com/en-us/iaas/tools/go-sdk-examples/latest/datasafe/GetCompatibleFormatsForSensitiveTypes.go.html to see an example of how to use GetCompatibleFormatsForSensitiveTypesRequest.
type GetCompatibleFormatsForSensitiveTypesRequest struct {

	// A filter to return only resources that match the specified compartment OCID.
	CompartmentId *string `mandatory:"true" contributesTo:"query" name:"compartmentId"`

	// Default is false.
	// When set to true, the hierarchy of compartments is traversed and all compartments and subcompartments in the tenancy are returned. Depends on the 'accessLevel' setting.
	CompartmentIdInSubtree *bool `mandatory:"false" contributesTo:"query" name:"compartmentIdInSubtree"`

	// Valid values are RESTRICTED and ACCESSIBLE. Default is RESTRICTED.
	// Setting this to ACCESSIBLE returns only those compartments for which the
	// user has INSPECT permissions directly or indirectly (permissions can be on a
	// resource in a subcompartment). When set to RESTRICTED permissions are checked and no partial results are displayed.
	AccessLevel GetCompatibleFormatsForSensitiveTypesAccessLevelEnum `mandatory:"false" contributesTo:"query" name:"accessLevel" omitEmpty:"true"`

	// For list pagination. The maximum number of items to return per page in a paginated "List" call. For details about how pagination works, see List Pagination (https://docs.cloud.oracle.com/en-us/iaas/Content/API/Concepts/usingapi.htm#nine).
	Limit *int `mandatory:"false" contributesTo:"query" name:"limit"`

	// For list pagination. The page token representing the page at which to start retrieving results. It is usually retrieved from a previous "List" call. For details about how pagination works, see List Pagination (https://docs.cloud.oracle.com/en-us/iaas/Content/API/Concepts/usingapi.htm#nine).
	Page *string `mandatory:"false" contributesTo:"query" name:"page"`

	// Unique identifier for the request.
	OpcRequestId *string `mandatory:"false" contributesTo:"header" name:"opc-request-id"`

	// Metadata about the request. This information will not be transmitted to the service, but
	// represents information that the SDK will consume to drive retry behavior.
	RequestMetadata common.RequestMetadata
}

func (request GetCompatibleFormatsForSensitiveTypesRequest) String() string {
	return common.PointerString(request)
}

// HTTPRequest implements the OCIRequest interface
func (request GetCompatibleFormatsForSensitiveTypesRequest) HTTPRequest(method, path string, binaryRequestBody *common.OCIReadSeekCloser, extraHeaders map[string]string) (http.Request, error) {

	_, err := request.ValidateEnumValue()
	if err != nil {
		return http.Request{}, err
	}
	return common.MakeDefaultHTTPRequestWithTaggedStructAndExtraHeaders(method, path, request, extraHeaders)
}

// BinaryRequestBody implements the OCIRequest interface
func (request GetCompatibleFormatsForSensitiveTypesRequest) BinaryRequestBody() (*common.OCIReadSeekCloser, bool) {

	return nil, false

}

// RetryPolicy implements the OCIRetryableRequest interface. This retrieves the specified retry policy.
func (request GetCompatibleFormatsForSensitiveTypesRequest) RetryPolicy() *common.RetryPolicy {
	return request.RequestMetadata.RetryPolicy
}

// ValidateEnumValue returns an error when providing an unsupported enum value
// This function is being called during constructing API request process
// Not recommended for calling this function directly
func (request GetCompatibleFormatsForSensitiveTypesRequest) ValidateEnumValue() (bool, error) {
	errMessage := []string{}
	if _, ok := GetMappingGetCompatibleFormatsForSensitiveTypesAccessLevelEnum(string(request.AccessLevel)); !ok && request.AccessLevel != "" {
		errMessage = append(errMessage, fmt.Sprintf("unsupported enum value for AccessLevel: %s. Supported values are: %s.", request.AccessLevel, strings.Join(GetGetCompatibleFormatsForSensitiveTypesAccessLevelEnumStringValues(), ",")))
	}
	if len(errMessage) > 0 {
		return true, fmt.Errorf(strings.Join(errMessage, "\n"))
	}
	return false, nil
}

// GetCompatibleFormatsForSensitiveTypesResponse wrapper for the GetCompatibleFormatsForSensitiveTypes operation
type GetCompatibleFormatsForSensitiveTypesResponse struct {

	// The underlying http response
	RawResponse *http.Response

	// A list of CompatibleFormatsForSensitiveTypes instances
	CompatibleFormatsForSensitiveTypes `presentIn:"body"`

	// Unique Oracle-assigned identifier for the request. If you need to contact Oracle about a particular request, please provide the request ID.
	OpcRequestId *string `presentIn:"header" name:"opc-request-id"`

	// For list pagination. When this header appears in the response, additional pages of results remain. Include opc-next-page value as the page parameter for the subsequent GET request to get the next batch of items. For details about how pagination works, see List Pagination (https://docs.cloud.oracle.com/iaas/Content/API/Concepts/usingapi.htm#nine).
	OpcNextPage *string `presentIn:"header" name:"opc-next-page"`

	// For pagination of a list of items. When paging through a list, if this header appears in the response,
	// then a partial list might have been returned. Include this value as the `page` parameter for the
	// subsequent GET request to get the previous batch of items.
	OpcPrevPage *string `presentIn:"header" name:"opc-prev-page"`
}

func (response GetCompatibleFormatsForSensitiveTypesResponse) String() string {
	return common.PointerString(response)
}

// HTTPResponse implements the OCIResponse interface
func (response GetCompatibleFormatsForSensitiveTypesResponse) HTTPResponse() *http.Response {
	return response.RawResponse
}

// GetCompatibleFormatsForSensitiveTypesAccessLevelEnum Enum with underlying type: string
type GetCompatibleFormatsForSensitiveTypesAccessLevelEnum string

// Set of constants representing the allowable values for GetCompatibleFormatsForSensitiveTypesAccessLevelEnum
const (
	GetCompatibleFormatsForSensitiveTypesAccessLevelRestricted GetCompatibleFormatsForSensitiveTypesAccessLevelEnum = "RESTRICTED"
	GetCompatibleFormatsForSensitiveTypesAccessLevelAccessible GetCompatibleFormatsForSensitiveTypesAccessLevelEnum = "ACCESSIBLE"
)

var mappingGetCompatibleFormatsForSensitiveTypesAccessLevelEnum = map[string]GetCompatibleFormatsForSensitiveTypesAccessLevelEnum{
	"RESTRICTED": GetCompatibleFormatsForSensitiveTypesAccessLevelRestricted,
	"ACCESSIBLE": GetCompatibleFormatsForSensitiveTypesAccessLevelAccessible,
}

var mappingGetCompatibleFormatsForSensitiveTypesAccessLevelEnumLowerCase = map[string]GetCompatibleFormatsForSensitiveTypesAccessLevelEnum{
	"restricted": GetCompatibleFormatsForSensitiveTypesAccessLevelRestricted,
	"accessible": GetCompatibleFormatsForSensitiveTypesAccessLevelAccessible,
}

// GetGetCompatibleFormatsForSensitiveTypesAccessLevelEnumValues Enumerates the set of values for GetCompatibleFormatsForSensitiveTypesAccessLevelEnum
func GetGetCompatibleFormatsForSensitiveTypesAccessLevelEnumValues() []GetCompatibleFormatsForSensitiveTypesAccessLevelEnum {
	values := make([]GetCompatibleFormatsForSensitiveTypesAccessLevelEnum, 0)
	for _, v := range mappingGetCompatibleFormatsForSensitiveTypesAccessLevelEnum {
		values = append(values, v)
	}
	return values
}

// GetGetCompatibleFormatsForSensitiveTypesAccessLevelEnumStringValues Enumerates the set of values in String for GetCompatibleFormatsForSensitiveTypesAccessLevelEnum
func GetGetCompatibleFormatsForSensitiveTypesAccessLevelEnumStringValues() []string {
	return []string{
		"RESTRICTED",
		"ACCESSIBLE",
	}
}

// GetMappingGetCompatibleFormatsForSensitiveTypesAccessLevelEnum performs case Insensitive comparison on enum value and return the desired enum
func GetMappingGetCompatibleFormatsForSensitiveTypesAccessLevelEnum(val string) (GetCompatibleFormatsForSensitiveTypesAccessLevelEnum, bool) {
	enum, ok := mappingGetCompatibleFormatsForSensitiveTypesAccessLevelEnumLowerCase[strings.ToLower(val)]
	return enum, ok
}
