// Copyright (c) 2016, 2018, 2023, Oracle and/or its affiliates.  All rights reserved.
// This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.
// Code generated. DO NOT EDIT.

package ocvp

import (
	"fmt"
	"github.com/oracle/oci-go-sdk/v65/common"
	"net/http"
	"strings"
)

// ListSupportedHostShapesRequest wrapper for the ListSupportedHostShapes operation
//
// # See also
//
// Click https://docs.cloud.oracle.com/en-us/iaas/tools/go-sdk-examples/latest/ocvp/ListSupportedHostShapes.go.html to see an example of how to use ListSupportedHostShapesRequest.
type ListSupportedHostShapesRequest struct {

	// The OCID (https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the compartment.
	CompartmentId *string `mandatory:"true" contributesTo:"query" name:"compartmentId"`

	// For list pagination. The maximum number of results per page, or items to return in a paginated
	// "List" call. For important details about how pagination works, see
	// List Pagination (https://docs.cloud.oracle.com/iaas/Content/API/Concepts/usingapi.htm#nine).
	Limit *int `mandatory:"false" contributesTo:"query" name:"limit"`

	// For list pagination. The value of the `opc-next-page` response header from the previous "List"
	// call. For important details about how pagination works, see
	// List Pagination (https://docs.cloud.oracle.com/iaas/Content/API/Concepts/usingapi.htm#nine).
	Page *string `mandatory:"false" contributesTo:"query" name:"page"`

	// Unique identifier for the request. If you need to contact Oracle about a particular
	// request, please provide the request ID.
	OpcRequestId *string `mandatory:"false" contributesTo:"header" name:"opc-request-id"`

	// A filter to return only resources that match the given name exactly.
	Name *string `mandatory:"false" contributesTo:"query" name:"name"`

	// A filter to return only resources that match the given SDDC type exactly.
	SddcType ListSupportedHostShapesSddcTypeEnum `mandatory:"false" contributesTo:"query" name:"sddcType" omitEmpty:"true"`

	// Metadata about the request. This information will not be transmitted to the service, but
	// represents information that the SDK will consume to drive retry behavior.
	RequestMetadata common.RequestMetadata
}

func (request ListSupportedHostShapesRequest) String() string {
	return common.PointerString(request)
}

// HTTPRequest implements the OCIRequest interface
func (request ListSupportedHostShapesRequest) HTTPRequest(method, path string, binaryRequestBody *common.OCIReadSeekCloser, extraHeaders map[string]string) (http.Request, error) {

	_, err := request.ValidateEnumValue()
	if err != nil {
		return http.Request{}, err
	}
	return common.MakeDefaultHTTPRequestWithTaggedStructAndExtraHeaders(method, path, request, extraHeaders)
}

// BinaryRequestBody implements the OCIRequest interface
func (request ListSupportedHostShapesRequest) BinaryRequestBody() (*common.OCIReadSeekCloser, bool) {

	return nil, false

}

// RetryPolicy implements the OCIRetryableRequest interface. This retrieves the specified retry policy.
func (request ListSupportedHostShapesRequest) RetryPolicy() *common.RetryPolicy {
	return request.RequestMetadata.RetryPolicy
}

// ValidateEnumValue returns an error when providing an unsupported enum value
// This function is being called during constructing API request process
// Not recommended for calling this function directly
func (request ListSupportedHostShapesRequest) ValidateEnumValue() (bool, error) {
	errMessage := []string{}
	if _, ok := GetMappingListSupportedHostShapesSddcTypeEnum(string(request.SddcType)); !ok && request.SddcType != "" {
		errMessage = append(errMessage, fmt.Sprintf("unsupported enum value for SddcType: %s. Supported values are: %s.", request.SddcType, strings.Join(GetListSupportedHostShapesSddcTypeEnumStringValues(), ",")))
	}
	if len(errMessage) > 0 {
		return true, fmt.Errorf(strings.Join(errMessage, "\n"))
	}
	return false, nil
}

// ListSupportedHostShapesResponse wrapper for the ListSupportedHostShapes operation
type ListSupportedHostShapesResponse struct {

	// The underlying http response
	RawResponse *http.Response

	// A list of SupportedHostShapeCollection instances
	SupportedHostShapeCollection `presentIn:"body"`

	// Unique Oracle-assigned identifier for the request. If you need to contact
	// Oracle about a particular request, please provide the request ID.
	OpcRequestId *string `presentIn:"header" name:"opc-request-id"`

	// For list pagination. When this header appears in the response, additional pages
	// of results remain. For important details about how pagination works, see
	// List Pagination (https://docs.cloud.oracle.com/iaas/Content/API/Concepts/usingapi.htm#nine).
	OpcNextPage *string `presentIn:"header" name:"opc-next-page"`
}

func (response ListSupportedHostShapesResponse) String() string {
	return common.PointerString(response)
}

// HTTPResponse implements the OCIResponse interface
func (response ListSupportedHostShapesResponse) HTTPResponse() *http.Response {
	return response.RawResponse
}

// ListSupportedHostShapesSddcTypeEnum Enum with underlying type: string
type ListSupportedHostShapesSddcTypeEnum string

// Set of constants representing the allowable values for ListSupportedHostShapesSddcTypeEnum
const (
	ListSupportedHostShapesSddcTypeProduction    ListSupportedHostShapesSddcTypeEnum = "PRODUCTION"
	ListSupportedHostShapesSddcTypeNonProduction ListSupportedHostShapesSddcTypeEnum = "NON_PRODUCTION"
)

var mappingListSupportedHostShapesSddcTypeEnum = map[string]ListSupportedHostShapesSddcTypeEnum{
	"PRODUCTION":     ListSupportedHostShapesSddcTypeProduction,
	"NON_PRODUCTION": ListSupportedHostShapesSddcTypeNonProduction,
}

var mappingListSupportedHostShapesSddcTypeEnumLowerCase = map[string]ListSupportedHostShapesSddcTypeEnum{
	"production":     ListSupportedHostShapesSddcTypeProduction,
	"non_production": ListSupportedHostShapesSddcTypeNonProduction,
}

// GetListSupportedHostShapesSddcTypeEnumValues Enumerates the set of values for ListSupportedHostShapesSddcTypeEnum
func GetListSupportedHostShapesSddcTypeEnumValues() []ListSupportedHostShapesSddcTypeEnum {
	values := make([]ListSupportedHostShapesSddcTypeEnum, 0)
	for _, v := range mappingListSupportedHostShapesSddcTypeEnum {
		values = append(values, v)
	}
	return values
}

// GetListSupportedHostShapesSddcTypeEnumStringValues Enumerates the set of values in String for ListSupportedHostShapesSddcTypeEnum
func GetListSupportedHostShapesSddcTypeEnumStringValues() []string {
	return []string{
		"PRODUCTION",
		"NON_PRODUCTION",
	}
}

// GetMappingListSupportedHostShapesSddcTypeEnum performs case Insensitive comparison on enum value and return the desired enum
func GetMappingListSupportedHostShapesSddcTypeEnum(val string) (ListSupportedHostShapesSddcTypeEnum, bool) {
	enum, ok := mappingListSupportedHostShapesSddcTypeEnumLowerCase[strings.ToLower(val)]
	return enum, ok
}
