// Copyright (c) 2016, 2018, 2023, Oracle and/or its affiliates.  All rights reserved.
// This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.
// Code generated. DO NOT EDIT.

// Application Dependency Management API
//
// Use the Application Dependency Management API to create knowledge bases and vulnerability audits.  For more information, see ADM (https://docs.cloud.oracle.com/Content/application-dependency-management/home.htm).
//

package adm

import (
	"encoding/json"
	"fmt"
	"github.com/oracle/oci-go-sdk/v65/common"
	"strings"
)

// CreateVulnerabilityAuditDetails Details to create a Vulnerability Audit.
type CreateVulnerabilityAuditDetails struct {

	// The Oracle Cloud identifier (OCID (https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm)) of the Knowledge Base.
	KnowledgeBaseId *string `mandatory:"true" json:"knowledgeBaseId"`

	// The type of the build tool.
	BuildType VulnerabilityAuditBuildTypeEnum `mandatory:"true" json:"buildType"`

	// The Oracle Cloud identifier (OCID (https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm)) of the compartment associated with the Vulnerability Audit.
	// If compartment identifier is not provided the compartment of the associated Knowledge Base will be used instead.
	CompartmentId *string `mandatory:"false" json:"compartmentId"`

	// List of Application Dependencies (without vulnerabilities).
	ApplicationDependencies []ApplicationDependency `mandatory:"false" json:"applicationDependencies"`

	Configuration *VulnerabilityAuditConfiguration `mandatory:"false" json:"configuration"`

	// The name of the Vulnerability Audit.
	DisplayName *string `mandatory:"false" json:"displayName"`

	Source VulnerabilityAuditSource `mandatory:"false" json:"source"`

	// Simple key-value pair that is applied without any predefined name, type or scope. Exists for cross-compatibility only.
	// Example: `{"bar-key": "value"}`
	FreeformTags map[string]string `mandatory:"false" json:"freeformTags"`

	// Defined tags for this resource. Each key is predefined and scoped to a namespace.
	// Example: `{"foo-namespace": {"bar-key": "value"}}`
	DefinedTags map[string]map[string]interface{} `mandatory:"false" json:"definedTags"`
}

func (m CreateVulnerabilityAuditDetails) String() string {
	return common.PointerString(m)
}

// ValidateEnumValue returns an error when providing an unsupported enum value
// This function is being called during constructing API request process
// Not recommended for calling this function directly
func (m CreateVulnerabilityAuditDetails) ValidateEnumValue() (bool, error) {
	errMessage := []string{}
	if _, ok := GetMappingVulnerabilityAuditBuildTypeEnum(string(m.BuildType)); !ok && m.BuildType != "" {
		errMessage = append(errMessage, fmt.Sprintf("unsupported enum value for BuildType: %s. Supported values are: %s.", m.BuildType, strings.Join(GetVulnerabilityAuditBuildTypeEnumStringValues(), ",")))
	}

	if len(errMessage) > 0 {
		return true, fmt.Errorf(strings.Join(errMessage, "\n"))
	}
	return false, nil
}

// UnmarshalJSON unmarshals from json
func (m *CreateVulnerabilityAuditDetails) UnmarshalJSON(data []byte) (e error) {
	model := struct {
		CompartmentId           *string                           `json:"compartmentId"`
		ApplicationDependencies []ApplicationDependency           `json:"applicationDependencies"`
		Configuration           *VulnerabilityAuditConfiguration  `json:"configuration"`
		DisplayName             *string                           `json:"displayName"`
		Source                  vulnerabilityauditsource          `json:"source"`
		FreeformTags            map[string]string                 `json:"freeformTags"`
		DefinedTags             map[string]map[string]interface{} `json:"definedTags"`
		KnowledgeBaseId         *string                           `json:"knowledgeBaseId"`
		BuildType               VulnerabilityAuditBuildTypeEnum   `json:"buildType"`
	}{}

	e = json.Unmarshal(data, &model)
	if e != nil {
		return
	}
	var nn interface{}
	m.CompartmentId = model.CompartmentId

	m.ApplicationDependencies = make([]ApplicationDependency, len(model.ApplicationDependencies))
	copy(m.ApplicationDependencies, model.ApplicationDependencies)
	m.Configuration = model.Configuration

	m.DisplayName = model.DisplayName

	nn, e = model.Source.UnmarshalPolymorphicJSON(model.Source.JsonData)
	if e != nil {
		return
	}
	if nn != nil {
		m.Source = nn.(VulnerabilityAuditSource)
	} else {
		m.Source = nil
	}

	m.FreeformTags = model.FreeformTags

	m.DefinedTags = model.DefinedTags

	m.KnowledgeBaseId = model.KnowledgeBaseId

	m.BuildType = model.BuildType

	return
}
