// Copyright (c) 2016, 2018, 2023, Oracle and/or its affiliates.  All rights reserved.
// This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.
// Code generated. DO NOT EDIT.

// Scanning API
//
// Use the Vulnerability Scanning Service (VSS) API to manage scan recipes, targets, and reports. For more information, see the Vulnerability Scanning documentation (https://docs.cloud.oracle.com/iaas/scanning/home.htm).
//

package vulnerabilityscanning

import (
	"fmt"
	"github.com/oracle/oci-go-sdk/v65/common"
	"strings"
)

// HostVulnerability A vulnerability found in one or more host scans in a compartment
type HostVulnerability struct {

	// ID of the vulnerability
	Id *string `mandatory:"true" json:"id"`

	// Compartment ID of the vulnerability. Vulnerabilities with identical CVEs can exist in different compartments, based on scan target location
	CompartmentId *string `mandatory:"true" json:"compartmentId"`

	// Severity of the vulnerability
	Severity ScanResultProblemSeverityEnum `mandatory:"true" json:"severity"`

	// Vulnerability name
	Name *string `mandatory:"true" json:"name"`

	// Date of scan result that most recently reported the vulnerability
	TimeLastDetected *common.SDKTime `mandatory:"true" json:"timeLastDetected"`

	// Date of scan result that first reported the vulnerability
	TimeFirstDetected *common.SDKTime `mandatory:"true" json:"timeFirstDetected"`

	// Number of scanned hosts in this compartment whose scan results currently report this vulnerability
	HostCount *int `mandatory:"true" json:"hostCount"`

	// Specifies the type of Vulnerability
	VulnerabilityType VulnerabilityTypeEnum `mandatory:"true" json:"vulnerabilityType"`

	// Mechanism used for scan authentication
	Authentication *string `mandatory:"false" json:"authentication"`

	// Reference to MITRE CVE ID
	CveReference *string `mandatory:"false" json:"cveReference"`

	CveDetails *CveDetails `mandatory:"false" json:"cveDetails"`

	// State of the vulnerability
	State ScanResultVulnerabilityStateEnum `mandatory:"false" json:"state,omitempty"`

	// The current state of the resource.
	LifecycleState LifecycleStateEnum `mandatory:"false" json:"lifecycleState,omitempty"`
}

func (m HostVulnerability) String() string {
	return common.PointerString(m)
}

// ValidateEnumValue returns an error when providing an unsupported enum value
// This function is being called during constructing API request process
// Not recommended for calling this function directly
func (m HostVulnerability) ValidateEnumValue() (bool, error) {
	errMessage := []string{}
	if _, ok := GetMappingScanResultProblemSeverityEnum(string(m.Severity)); !ok && m.Severity != "" {
		errMessage = append(errMessage, fmt.Sprintf("unsupported enum value for Severity: %s. Supported values are: %s.", m.Severity, strings.Join(GetScanResultProblemSeverityEnumStringValues(), ",")))
	}
	if _, ok := GetMappingVulnerabilityTypeEnum(string(m.VulnerabilityType)); !ok && m.VulnerabilityType != "" {
		errMessage = append(errMessage, fmt.Sprintf("unsupported enum value for VulnerabilityType: %s. Supported values are: %s.", m.VulnerabilityType, strings.Join(GetVulnerabilityTypeEnumStringValues(), ",")))
	}

	if _, ok := GetMappingScanResultVulnerabilityStateEnum(string(m.State)); !ok && m.State != "" {
		errMessage = append(errMessage, fmt.Sprintf("unsupported enum value for State: %s. Supported values are: %s.", m.State, strings.Join(GetScanResultVulnerabilityStateEnumStringValues(), ",")))
	}
	if _, ok := GetMappingLifecycleStateEnum(string(m.LifecycleState)); !ok && m.LifecycleState != "" {
		errMessage = append(errMessage, fmt.Sprintf("unsupported enum value for LifecycleState: %s. Supported values are: %s.", m.LifecycleState, strings.Join(GetLifecycleStateEnumStringValues(), ",")))
	}
	if len(errMessage) > 0 {
		return true, fmt.Errorf(strings.Join(errMessage, "\n"))
	}
	return false, nil
}
